# NumericAnnex

NumericAnnex supplements the numeric facilities provided in the Swift standard
library.


## Features

- [x] `Math`, a protocol for types that provides square root, cube root, and
      elementary transcendental functions.
- [x] `FloatingPointMath`, a protocol for floating point types that provides a
      selection of special functions such as the gamma function.
- [x] `Complex`, a value type to represent complex values in Cartesian form,
      which provides all functions required by `Math`.
- [x] `Rational`, a value type to represent rational values, which supports the
      special values infinity, negative infinity, and NaN ("not a number").

> Note: This project is in the early stages of development and is not
> production-ready at this time.


## Requirements

NumericAnnex now requires the latest development (master) branch of Swift or a
recent development snapshot that includes the revised numeric protocols. It
requires either `Darwin` or `Glibc` for transcendental functions provided by the
C standard library.


## Installation

After NumericAnnex has been cloned or downloaded locally, build the library by
invoking `swift build`, or run tests with `swift test`. An Xcode project can be
generated by invoking `swift package generate-xcodeproj`.

[Swift Package Manager](https://swift.org/package-manager/) can also be used to
add the package as a dependency for your own project. See Swift documentation
for details.


## Documentation

All public protocols, types, and functions have been carefully documented in
comments.

In general, the project adheres to design patterns found in the Swift standard
library. For example, `FloatingPoint` types provide a method named
`squareRoot()`. Therefore, NumericAnnex `Math` types provide methods such as
`cubeRoot()` and `inverseHyperbolicTangent()`.

Note, however, that NumericAnnex does not declare new free functions. Instead,
functions such as `cbrt(_:)` and `atanh(_:)` are provided as static members.
This avoids potential collisions with C standard library functions that you may
wish. It also promotes clarity at the use site that can be invaluable, for
example, when the result of a complex operation differs from that of its real
counterpart (`Complex128.cbrt(-8) != -2`). If the type of an expression can be
unambiguously inferred, Swift permits the use of a terser notation, if desired
(`_ = 2.5 + .cbrt(3)`).


## Future directions

- [ ] Add more tests
- [ ] Design and implement `BigInt`
- [ ] Design and implement `Random`


## License

All original work is released under the MIT license. See
[LICENSE](https://github.com/xwu/NumericAnnex/blob/master/LICENSE) for details.

Portions of the complex square root and elementary transcendental functions use
checks for special values adapted from libc++. Code in libc++ is dual-licensed
under the MIT and UIUC/NCSA licenses.
